/*	Gordon Williams
	July 6, 2000
	This example shows a module and function doc strings
	as well as how to create an exception.  A "None" return
	value is created in a number of different ways.

	print example.__doc__ shows module doc string
	print example.add.__doc__ shows function doc string for add
	example.myExcept() produces a trace back with the new exception
	example.returnNone()s show that a Python None can be returned*/

#include "Python.h"


PyObject *exception = NULL;

static PyObject * example_returnNone1(PyObject *self, PyObject *args)
{
	return Py_BuildValue("");
};

static PyObject * example_returnNone2(PyObject *self, PyObject *args)
{
	PyObject *t;
	t= Py_None;
	Py_INCREF(t);
	return t;
};

static PyObject * example_returnNone3(PyObject *self, PyObject *args)
{
	PyObject *t;
	t= Py_BuildValue("");
	return t;
};

static PyObject * example_returnNone4(PyObject *self, PyObject *args)
{
	PyObject *t;
	t= Py_None;
	return Py_BuildValue("(O,O,O)",t,t,t);
};


static PyObject * example_add(PyObject *self, PyObject *args)
{
	int m, n;
	if (!PyArg_ParseTuple(args, "ii", &m, &n))
	{
		return NULL;
	}
	return Py_BuildValue("i", m+n);
};

static PyObject * example_myExcept(PyObject *self, PyObject *args)
{
	PyErr_SetString(exception, "Made an error and caused exception");
	return NULL;
};
	
static PyMethodDef example_methods[] =
{
	{"add", example_add, 1, "This function adds two ints"},
	{"myExcept", example_myExcept, 1, "This function test exceptions."},
	{"returnNone1",example_returnNone1, 1, "Returns None method 1"},
	{"returnNone2",example_returnNone2, 1, "Returns None method 2"},
	{"returnNone3",example_returnNone3, 1, "Returns None method 3"},
	{"returnNone4",example_returnNone4, 1, "Returns None method 4"},
	{NULL, NULL}
};

__declspec(dllexport)

void example_init()
{
	PyObject *m, *d;

	static char example__doc__[] =
	"This module contains a bunch of stuff for an example";

    	m = Py_InitModule3( "example", example_methods, example__doc__);
	d = PyModule_GetDict(m);
	exception = PyErr_NewException("example.error", NULL, NULL);
	PyDict_SetItemString(d, "error", exception);
};
	
